@@ -2,21 +2,21 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
-use crate::{
-    format::datetime,
-    options::DateTimeFormatOptions,
-    provider::{
-        gregory::{DatePatternsV1Marker, DateSymbolsV1Marker},
-        helpers::DateTimePatterns,
-    },
-};
-use alloc::string::String;
-use icu_locid::Locale;
-use icu_provider::prelude::*;
+//! The collection of code that is needed for handling formatting operations for DateTimes.
+//! Central to this is the [`DateTimeFormat`].
 
 use crate::{
-    date::DateTimeInput, pattern::Pattern, provider, DateTimeFormatError, FormattedDateTime,
+    options::{components, DateTimeFormatOptions},
+    provider::calendar::{DatePatternsV1Marker, DateSkeletonPatternsV1Marker, DateSymbolsV1Marker},
+    raw,
 };
+use alloc::string::String;
+use core::marker::PhantomData;
+use icu_locid::Locale;
+use icu_plurals::provider::PluralRulesV1Marker;
+use icu_provider::prelude::*;
+
+use crate::{date::DateTimeInput, CldrCalendar, DateTimeFormatError, FormattedDateTime};
 
 /// [`DateTimeFormat`] is the main structure of the [`icu_datetime`] component.
 /// When constructed, it uses data from the [`DataProvider`], selected [`Locale`] and provided options to
@@ -34,7 +34,7 @@ use crate::{
 /// use icu::locid::Locale;
 /// use icu::locid::macros::langid;
 /// use icu::datetime::{DateTimeFormat, options::length};
-/// use icu::datetime::mock::datetime::MockDateTime;
+/// use icu::calendar::{DateTime, Gregorian};
 /// use icu_provider::inv::InvariantDataProvider;
 ///
 /// let locale: Locale = langid!("en").into();
@@ -46,11 +46,11 @@ use crate::{
 ///     time: Some(length::Time::Short),
 ///     ..Default::default()
 /// };
-/// let dtf = DateTimeFormat::try_new(locale, &provider, &options.into())
+/// let dtf = DateTimeFormat::<Gregorian>::try_new(locale, &provider, &options.into())
 ///     .expect("Failed to create DateTimeFormat instance.");
 ///
 ///
-/// let datetime = MockDateTime::try_new(2020, 9, 1, 12, 34, 28)
+/// let datetime = DateTime::new_gregorian_datetime_from_integers(2020, 9, 1, 12, 34, 28)
 ///     .expect("Failed to construct DateTime.");
 ///
 /// let value = dtf.format_to_string(&datetime);
@@ -58,13 +58,9 @@ use crate::{
 ///
 /// This model replicates that of `ICU` and `ECMA402`. In the future this will become even more pronounced
 /// when we introduce asynchronous [`DataProvider`] and corresponding asynchronous constructor.
-pub struct DateTimeFormat<'data> {
-    pub(super) locale: Locale,
-    pub(super) pattern: Pattern,
-    pub(super) symbols: Option<DataPayload<'data, DateSymbolsV1Marker>>,
-}
+pub struct DateTimeFormat<C>(pub(super) raw::DateTimeFormat, PhantomData<C>);
 
-impl<'data> DateTimeFormat<'data> {
+impl<C: CldrCalendar> DateTimeFormat<C> {
     /// Constructor that takes a selected [`Locale`], reference to a [`DataProvider`] and
     /// a list of options, then collects all data necessary to format date and time values into the given locale.
     ///
@@ -73,8 +69,8 @@ impl<'data> DateTimeFormat<'data> {
     /// ```
     /// use icu::locid::Locale;
     /// use icu::locid::macros::langid;
+    /// use icu::calendar::Gregorian;
     /// use icu::datetime::{DateTimeFormat, DateTimeFormatOptions};
-    /// use icu::datetime::mock::datetime::MockDateTime;
     /// use icu_provider::inv::InvariantDataProvider;
     ///
     /// let locale: Locale = langid!("en").into();
@@ -83,89 +79,26 @@ impl<'data> DateTimeFormat<'data> {
     ///
     /// let options = DateTimeFormatOptions::default();
     ///
-    /// let dtf = DateTimeFormat::try_new(locale, &provider, &options);
+    /// let dtf = DateTimeFormat::<Gregorian>::try_new(locale, &provider, &options);
     ///
     /// assert_eq!(dtf.is_ok(), true);
     /// ```
-    pub fn try_new<
-        T: Into<Locale>,
-        D: DataProvider<'data, DateSymbolsV1Marker>
-            + DataProvider<'data, DatePatternsV1Marker>
-            + ?Sized,
-    >(
+    #[inline]
+    pub fn try_new<T: Into<Locale>, D>(
         locale: T,
         data_provider: &D,
         options: &DateTimeFormatOptions,
-    ) -> Result<Self, DateTimeFormatError> {
-        let locale = locale.into();
-
-        let patterns_data: icu_provider::DataPayload<'_, provider::gregory::DatePatternsV1Marker> =
-            data_provider
-                .load_payload(&DataRequest {
-                    resource_path: ResourcePath {
-                        key: provider::key::GREGORY_DATE_PATTERNS_V1,
-                        options: ResourceOptions {
-                            variant: None,
-                            langid: Some(locale.clone().into()),
-                        },
-                    },
-                })?
-                .take_payload()?;
-
-        let pattern = patterns_data
-            .get()
-            .get_pattern_for_options(options)?
-            .unwrap_or_default();
-
-        let requires_data = datetime::analyze_pattern(&pattern, false)
-            .map_err(|field| DateTimeFormatError::UnsupportedField(field.symbol))?;
-
-        let symbols_data = if requires_data {
-            Some(
-                data_provider
-                    .load_payload(&DataRequest {
-                        resource_path: ResourcePath {
-                            key: provider::key::GREGORY_DATE_SYMBOLS_V1,
-                            options: ResourceOptions {
-                                variant: None,
-                                langid: Some(locale.clone().into()),
-                            },
-                        },
-                    })?
-                    .take_payload()?,
-            )
-        } else {
-            None
-        };
-
-        Ok(Self::new(locale, pattern, symbols_data))
-    }
-
-    /// Creates a new [`DateTimeFormat`] regardless of whether there are time-zone symbols in the pattern.
-    ///
-    /// By contrast, the public [`DateTimeFormat::try_new()`] function will return an error if there are
-    /// time-zone symbols in the pattern.
-    ///
-    /// This function is only `pub(super)` (not `pub`) because it is needed by [`ZonedDateTimeFormat`]
-    /// to create a [`DateTimeFormat`] for use internally. [`ZonedDateTimeFormat`] maintains
-    /// the invariant that [`DateTimeFormat`] will not be used to format the time zone.
-    ///
-    /// Creating a [`DateTimeFormat`] with time-zone symbols should always be an error
-    /// in public contexts.
-    ///
-    /// [`ZonedDateTimeFormat`]: crate::zoned_datetime::ZonedDateTimeFormat
-    pub(super) fn new<T: Into<Locale>>(
-        locale: T,
-        pattern: Pattern,
-        symbols: Option<DataPayload<'data, DateSymbolsV1Marker>>,
-    ) -> Self {
-        let locale = locale.into();
-
-        Self {
-            locale,
-            pattern,
-            symbols,
-        }
+    ) -> Result<Self, DateTimeFormatError>
+    where
+        D: DataProvider<DateSymbolsV1Marker>
+            + DataProvider<DatePatternsV1Marker>
+            + DataProvider<DateSkeletonPatternsV1Marker>
+            + DataProvider<PluralRulesV1Marker>,
+    {
+        Ok(Self(
+            raw::DateTimeFormat::try_new(locale, data_provider, options, C::IDENTIFIER)?,
+            PhantomData,
+        ))
     }
 
     /// Takes a [`DateTimeInput`] implementer and returns an instance of a [`FormattedDateTime`]
@@ -177,15 +110,15 @@ impl<'data> DateTimeFormat<'data> {
     /// use icu::locid::Locale;
     /// use icu::locid::macros::langid;
     /// use icu::datetime::{DateTimeFormat, DateTimeFormatOptions};
-    /// use icu::datetime::mock::datetime::MockDateTime;
+    /// use icu::calendar::{DateTime, Gregorian};
     /// use icu_provider::inv::InvariantDataProvider;
     /// # let locale: Locale = langid!("en").into();
     /// # let provider = InvariantDataProvider;
     /// # let options = DateTimeFormatOptions::default();
-    /// let dtf = DateTimeFormat::try_new(locale, &provider, &options)
+    /// let dtf = DateTimeFormat::<Gregorian>::try_new(locale, &provider, &options)
     ///     .expect("Failed to create DateTimeFormat instance.");
     ///
-    /// let datetime = MockDateTime::try_new(2020, 9, 1, 12, 34, 28)
+    /// let datetime = DateTime::new_gregorian_datetime_from_integers(2020, 9, 1, 12, 34, 28)
     ///     .expect("Failed to construct DateTime.");
     ///
     /// let formatted_date = dtf.format(&datetime);
@@ -196,16 +129,12 @@ impl<'data> DateTimeFormat<'data> {
     /// At the moment, there's little value in using that over one of the other `format` methods,
     /// but [`FormattedDateTime`] will grow with methods for iterating over fields, extracting information
     /// about formatted date and so on.
+    #[inline]
     pub fn format<'l, T>(&'l self, value: &'l T) -> FormattedDateTime<'l, T>
     where
-        T: DateTimeInput,
+        T: DateTimeInput<Calendar = C>,
     {
-        FormattedDateTime {
-            pattern: &self.pattern,
-            symbols: self.symbols.as_ref().map(|s| s.get()),
-            datetime: value,
-            locale: &self.locale,
-        }
+        self.0.format(value)
     }
 
     /// Takes a mutable reference to anything that implements [`Write`](std::fmt::Write) trait
@@ -217,15 +146,15 @@ impl<'data> DateTimeFormat<'data> {
     /// use icu::locid::Locale;
     /// use icu::locid::macros::langid;
     /// use icu::datetime::{DateTimeFormat, DateTimeFormatOptions};
-    /// use icu::datetime::mock::datetime::MockDateTime;
+    /// use icu::calendar::{DateTime, Gregorian};
     /// use icu_provider::inv::InvariantDataProvider;
     /// # let locale: Locale = langid!("en").into();
     /// # let provider = InvariantDataProvider;
     /// # let options = DateTimeFormatOptions::default();
-    /// let dtf = DateTimeFormat::try_new(locale, &provider, &options.into())
+    /// let dtf = DateTimeFormat::<Gregorian>::try_new(locale, &provider, &options.into())
     ///     .expect("Failed to create DateTimeFormat instance.");
     ///
-    /// let datetime = MockDateTime::try_new(2020, 9, 1, 12, 34, 28)
+    /// let datetime = DateTime::new_gregorian_datetime_from_integers(2020, 9, 1, 12, 34, 28)
     ///     .expect("Failed to construct DateTime.");
     ///
     /// let mut buffer = String::new();
@@ -234,19 +163,13 @@ impl<'data> DateTimeFormat<'data> {
     ///
     /// let _ = format!("Date: {}", buffer);
     /// ```
+    #[inline]
     pub fn format_to_write(
         &self,
         w: &mut impl core::fmt::Write,
-        value: &impl DateTimeInput,
+        value: &impl DateTimeInput<Calendar = C>,
     ) -> core::fmt::Result {
-        datetime::write_pattern(
-            &self.pattern,
-            self.symbols.as_ref().map(|s| s.get()),
-            value,
-            &self.locale,
-            w,
-        )
-        .map_err(|_| core::fmt::Error)
+        self.0.format_to_write(w, value)
     }
 
     /// Takes a [`DateTimeInput`] implementer and returns it formatted as a string.
@@ -257,23 +180,62 @@ impl<'data> DateTimeFormat<'data> {
     /// use icu::locid::Locale;
     /// use icu::locid::macros::langid;
     /// use icu::datetime::{DateTimeFormat, DateTimeFormatOptions};
-    /// use icu::datetime::mock::datetime::MockDateTime;
+    /// use icu::calendar::{DateTime, Gregorian};
     /// use icu_provider::inv::InvariantDataProvider;
     /// # let locale: Locale = langid!("en").into();
     /// # let provider = InvariantDataProvider;
     /// # let options = DateTimeFormatOptions::default();
-    /// let dtf = DateTimeFormat::try_new(locale, &provider, &options.into())
+    /// let dtf = DateTimeFormat::<Gregorian>::try_new(locale, &provider, &options.into())
     ///     .expect("Failed to create DateTimeFormat instance.");
     ///
-    /// let datetime = MockDateTime::try_new(2020, 9, 1, 12, 34, 28)
+    /// let datetime = DateTime::new_gregorian_datetime_from_integers(2020, 9, 1, 12, 34, 28)
     ///     .expect("Failed to construct DateTime.");
     ///
     /// let _ = dtf.format_to_string(&datetime);
     /// ```
-    pub fn format_to_string(&self, value: &impl DateTimeInput) -> String {
-        let mut s = String::new();
-        self.format_to_write(&mut s, value)
-            .expect("Failed to write to a String.");
-        s
+    #[inline]
+    pub fn format_to_string(&self, value: &impl DateTimeInput<Calendar = C>) -> String {
+        self.0.format_to_string(value)
+    }
+
+    /// Returns a [`components::Bag`] that represents the resolved components for the
+    /// options that were provided to the [`DateTimeFormat`]. The developer may request
+    /// a certain set of options for a [`DateTimeFormat`] but the locale and resolution
+    /// algorithm may change certain details of what actually gets resolved.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu::calendar::Gregorian;
+    /// use icu::datetime::{
+    ///     options::{components, length},
+    ///     DateTimeFormat, DateTimeFormatOptions,
+    /// };
+    /// use icu::locid::macros::langid;
+    /// use icu::locid::Locale;
+    ///
+    /// let options = DateTimeFormatOptions::Length(length::Bag {
+    ///     date: Some(length::Date::Medium),
+    ///     time: None,
+    ///     preferences: None,
+    /// });
+    ///
+    /// let locale: Locale = langid!("en").into();
+    /// let provider = icu_testdata::get_provider();
+    /// let dtf = DateTimeFormat::<Gregorian>::try_new(locale, &provider, &options)
+    ///     .expect("Failed to create DateTimeFormat instance.");
+    ///
+    /// assert_eq!(
+    ///     dtf.resolve_components(),
+    ///     components::Bag {
+    ///         year: Some(components::Year::Numeric),
+    ///         month: Some(components::Month::Short),
+    ///         day: Some(components::Numeric::Numeric),
+    ///         ..Default::default()
+    ///     }
+    /// );
+    /// ```
+    pub fn resolve_components(&self) -> components::Bag {
+        self.0.resolve_components()
     }
 }
