@@ -2,8 +2,11 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
+#[macro_use]
+mod macros;
 mod length;
 pub(crate) mod symbols;
+mod ule;
 
 use displaydoc::Display;
 pub use length::{FieldLength, LengthError};
@@ -11,7 +14,7 @@ pub use symbols::*;
 
 use core::{
     cmp::{Ord, PartialOrd},
-    convert::{TryFrom, TryInto},
+    convert::TryFrom,
 };
 
 #[derive(Display, Debug)]
@@ -34,10 +37,12 @@ pub struct Field {
 }
 
 impl Field {
-    pub fn get_length_type(&self) -> TextOrNumeric {
+    pub(crate) fn get_length_type(&self) -> TextOrNumeric {
         match self.symbol {
+            FieldSymbol::Era => TextOrNumeric::Text,
             FieldSymbol::Year(year) => year.get_length_type(self.length),
             FieldSymbol::Month(month) => month.get_length_type(self.length),
+            FieldSymbol::Week(week) => week.get_length_type(self.length),
             FieldSymbol::Day(day) => day.get_length_type(self.length),
             FieldSymbol::Weekday(weekday) => weekday.get_length_type(self.length),
             FieldSymbol::DayPeriod(day_period) => day_period.get_length_type(self.length),
@@ -47,6 +52,11 @@ impl Field {
             FieldSymbol::TimeZone(zone) => zone.get_length_type(self.length),
         }
     }
+
+    #[inline]
+    pub(crate) fn bytes_in_range(symbol: &u8, length: &u8) -> bool {
+        FieldSymbol::idx_in_range(symbol) && FieldLength::idx_in_range(length)
+    }
 }
 
 impl From<(FieldSymbol, FieldLength)> for Field {
@@ -63,9 +73,7 @@ impl TryFrom<(FieldSymbol, usize)> for Field {
     fn try_from(input: (FieldSymbol, usize)) -> Result<Self, Self::Error> {
         let (symbol, length) = (
             input.0,
-            input
-                .1
-                .try_into()
+            FieldLength::from_idx(input.1 as u8)
                 .map_err(|_| Self::Error::InvalidLength(input.0))?,
         );
         Ok(Self { symbol, length })
