@@ -2,17 +2,22 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
-use crate::arithmetic;
 use crate::date::{DateTimeInput, DateTimeInputWithLocale, LocalizedDateTimeInput};
 use crate::error::DateTimeFormatError as Error;
-use crate::fields::{self, Field, FieldLength, FieldSymbol};
-use crate::pattern::{Pattern, PatternItem};
+use crate::fields::{self, Field, FieldLength, FieldSymbol, Week, Year};
+use crate::pattern::{
+    runtime::{Pattern, PatternPlurals},
+    PatternItem,
+};
 use crate::provider;
-use crate::provider::helpers::DateTimeSymbols;
+use crate::provider::calendar::patterns::PatternPluralsFromPatternsV1Marker;
+use crate::provider::date_time::DateTimeSymbols;
 
 use alloc::string::ToString;
 use core::fmt;
 use icu_locid::Locale;
+use icu_plurals::PluralRules;
+use icu_provider::DataPayload;
 use writeable::Writeable;
 
 /// [`FormattedDateTime`] is a intermediate structure which can be retrieved as
@@ -28,15 +33,15 @@ use writeable::Writeable;
 /// use icu::locid::Locale;
 /// use icu::locid::macros::langid;
 /// use icu::datetime::{DateTimeFormat, DateTimeFormatOptions};
-/// use icu::datetime::mock::datetime::MockDateTime;
+/// use icu::calendar::{DateTime, Gregorian};
 /// use icu_provider::inv::InvariantDataProvider;
 /// let locale: Locale = langid!("en").into();
 /// # let provider = InvariantDataProvider;
 /// # let options = DateTimeFormatOptions::default();
-/// let dtf = DateTimeFormat::try_new(locale, &provider, &options)
+/// let dtf = DateTimeFormat::<Gregorian>::try_new(locale, &provider, &options)
 ///     .expect("Failed to create DateTimeFormat instance.");
 ///
-/// let datetime = MockDateTime::try_new(2020, 9, 1, 12, 34, 28)
+/// let datetime = DateTime::new_gregorian_datetime_from_integers(2020, 9, 1, 12, 34, 28)
 ///     .expect("Failed to construct DateTime.");
 ///
 /// let formatted_date = dtf.format(&datetime);
@@ -47,10 +52,11 @@ pub struct FormattedDateTime<'l, T>
 where
     T: DateTimeInput,
 {
-    pub(crate) pattern: &'l Pattern,
-    pub(crate) symbols: Option<&'l provider::gregory::DateSymbolsV1>,
+    pub(crate) patterns: &'l DataPayload<PatternPluralsFromPatternsV1Marker>,
+    pub(crate) symbols: Option<&'l provider::calendar::DateSymbolsV1<'l>>,
     pub(crate) datetime: &'l T,
     pub(crate) locale: &'l Locale,
+    pub(crate) ordinal_rules: Option<&'l PluralRules>,
 }
 
 impl<'l, T> Writeable for FormattedDateTime<'l, T>
@@ -58,8 +64,15 @@ where
     T: DateTimeInput,
 {
     fn write_to<W: fmt::Write + ?Sized>(&self, sink: &mut W) -> fmt::Result {
-        write_pattern(self.pattern, self.symbols, self.datetime, self.locale, sink)
-            .map_err(|_| core::fmt::Error)
+        write_pattern_plurals(
+            &self.patterns.get().0,
+            self.symbols,
+            self.datetime,
+            self.ordinal_rules,
+            self.locale,
+            sink,
+        )
+        .map_err(|_| core::fmt::Error)
     }
 
     // TODO(#489): Implement write_len
@@ -70,8 +83,7 @@ where
     T: DateTimeInput,
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write_pattern(self.pattern, self.symbols, self.datetime, self.locale, f)
-            .map_err(|_| core::fmt::Error)
+        self.write_to(f)
     }
 }
 
@@ -95,10 +107,30 @@ where
     }
 }
 
-pub fn write_pattern<T, W>(
-    pattern: &crate::pattern::Pattern,
-    symbols: Option<&provider::gregory::DateSymbolsV1>,
+fn write_pattern<T, W>(
+    pattern: &crate::pattern::runtime::Pattern,
+    symbols: Option<&provider::calendar::DateSymbolsV1>,
+    loc_datetime: &impl LocalizedDateTimeInput<T>,
+    w: &mut W,
+) -> Result<(), Error>
+where
+    T: DateTimeInput,
+    W: fmt::Write + ?Sized,
+{
+    for item in pattern.items.iter() {
+        match item {
+            PatternItem::Field(field) => write_field(pattern, field, symbols, loc_datetime, w)?,
+            PatternItem::Literal(ch) => w.write_char(ch)?,
+        }
+    }
+    Ok(())
+}
+
+pub fn write_pattern_plurals<T, W>(
+    patterns: &PatternPlurals,
+    symbols: Option<&provider::calendar::DateSymbolsV1>,
     datetime: &T,
+    ordinal_rules: Option<&PluralRules>,
     locale: &Locale,
     w: &mut W,
 ) -> Result<(), Error>
@@ -107,13 +139,8 @@ where
     W: fmt::Write + ?Sized,
 {
     let loc_datetime = DateTimeInputWithLocale::new(datetime, locale);
-    for item in pattern.items() {
-        match item {
-            PatternItem::Field(field) => write_field(pattern, field, symbols, &loc_datetime, w)?,
-            PatternItem::Literal(l) => w.write_str(l)?,
-        }
-    }
-    Ok(())
+    let pattern = patterns.select(&loc_datetime, ordinal_rules)?;
+    write_pattern(pattern, symbols, &loc_datetime, w)
 }
 
 // This function assumes that the correct decision has been
@@ -122,9 +149,9 @@ where
 // When modifying the list of fields using symbols,
 // update the matching query in `analyze_pattern` function.
 pub(super) fn write_field<T, W>(
-    pattern: &crate::pattern::Pattern,
-    field: &fields::Field,
-    symbols: Option<&crate::provider::gregory::DateSymbolsV1>,
+    pattern: &crate::pattern::runtime::Pattern,
+    field: fields::Field,
+    symbols: Option<&crate::provider::calendar::DateSymbolsV1>,
     datetime: &impl LocalizedDateTimeInput<T>,
     w: &mut W,
 ) -> Result<(), Error>
@@ -133,15 +160,32 @@ where
     W: fmt::Write + ?Sized,
 {
     match field.symbol {
-        FieldSymbol::Year(..) => format_number(
-            w,
-            datetime
-                .datetime()
-                .year()
-                .ok_or(Error::MissingInputField)?
-                .number as isize,
-            field.length,
-        )?,
+        FieldSymbol::Era => {
+            let symbol = symbols
+                .expect("Expect symbols to be present")
+                .get_symbol_for_era(
+                    field.length,
+                    &datetime
+                        .datetime()
+                        .year()
+                        .ok_or(Error::MissingInputField)?
+                        .era
+                        .0,
+                )?;
+            w.write_str(symbol)?
+        }
+        FieldSymbol::Year(year) => match year {
+            Year::Calendar => format_number(
+                w,
+                datetime
+                    .datetime()
+                    .year()
+                    .ok_or(Error::MissingInputField)?
+                    .number as isize,
+                field.length,
+            )?,
+            Year::WeekOf => format_number(w, datetime.year_week()?.number as isize, field.length)?,
+        },
         FieldSymbol::Month(month) => match field.length {
             FieldLength::One | FieldLength::TwoDigit => format_number(
                 w,
@@ -164,10 +208,14 @@ where
                             .ok_or(Error::MissingInputField)?
                             .number as usize
                             - 1,
-                    );
+                    )?;
                 w.write_str(symbol)?
             }
         },
+        FieldSymbol::Week(Week::WeekOfYear) => {
+            format_number(w, datetime.week_of_year()?.0 as isize, field.length)?
+        }
+        field @ FieldSymbol::Week(_) => return Err(Error::UnsupportedField(field)),
         FieldSymbol::Weekday(weekday) => {
             let dow = datetime
                 .datetime()
@@ -175,7 +223,7 @@ where
                 .ok_or(Error::MissingInputField)?;
             let symbol = symbols
                 .expect("Expect symbols to be present")
-                .get_symbol_for_weekday(weekday, field.length, dow);
+                .get_symbol_for_weekday(weekday, field.length, dow)?;
             w.write_str(symbol)?
         }
         FieldSymbol::Day(..) => format_number(
@@ -238,12 +286,11 @@ where
                     period,
                     field.length,
                     datetime.datetime().hour().ok_or(Error::MissingInputField)?,
-                    arithmetic::is_top_of_hour(
-                        pattern,
+                    pattern.time_granularity.is_top_of_hour(
                         datetime.datetime().minute().map(u8::from).unwrap_or(0),
                         datetime.datetime().second().map(u8::from).unwrap_or(0),
                     ),
-                );
+                )?;
             w.write_str(symbol)?
         }
         field @ FieldSymbol::TimeZone(_) => return Err(Error::UnsupportedField(field)),
@@ -253,8 +300,8 @@ where
 
 // This function determins whether the struct will load symbols data.
 // Keep it in sync with the `write_field` use of symbols.
-pub fn analyze_pattern(pattern: &Pattern, supports_time_zones: bool) -> Result<bool, &Field> {
-    let fields = pattern.items().iter().filter_map(|p| match p {
+pub fn analyze_pattern(pattern: &Pattern, supports_time_zones: bool) -> Result<bool, Field> {
+    let fields = pattern.items.iter().filter_map(|p| match p {
         PatternItem::Field(field) => Some(field),
         _ => None,
     });
@@ -264,6 +311,7 @@ pub fn analyze_pattern(pattern: &Pattern, supports_time_zones: bool) -> Result<b
     for field in fields {
         if !requires_symbols {
             requires_symbols = match field.symbol {
+                FieldSymbol::Era => true,
                 FieldSymbol::Month(_) => {
                     !matches!(field.length, FieldLength::One | FieldLength::TwoDigit)
                 }
@@ -288,6 +336,16 @@ pub fn analyze_pattern(pattern: &Pattern, supports_time_zones: bool) -> Result<b
     Ok(requires_symbols)
 }
 
+// This function determines whether any patterns will load symbols data.
+pub fn analyze_patterns(
+    patterns: &PatternPlurals,
+    supports_time_zones: bool,
+) -> Result<bool, Field> {
+    patterns.patterns_iter().try_fold(false, |a, pattern| {
+        analyze_pattern(pattern, supports_time_zones).map(|b| a || b)
+    })
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -295,17 +353,17 @@ mod tests {
     #[test]
     #[cfg(feature = "provider_serde")]
     fn test_basic() {
-        use crate::mock::datetime::MockDateTime;
-        use crate::provider::gregory::DateSymbolsV1Marker;
+        use crate::provider::calendar::DateSymbolsV1Marker;
+        use icu_calendar::DateTime;
         use icu_provider::prelude::*;
 
         let provider = icu_testdata::get_provider();
         let data: DataPayload<DateSymbolsV1Marker> = provider
             .load_payload(&DataRequest {
                 resource_path: ResourcePath {
-                    key: provider::key::GREGORY_DATE_SYMBOLS_V1,
+                    key: provider::key::DATE_SYMBOLS_V1,
                     options: ResourceOptions {
-                        variant: None,
+                        variant: Some("gregory".into()),
                         langid: Some("en".parse().unwrap()),
                     },
                 },
@@ -313,17 +371,12 @@ mod tests {
             .unwrap()
             .take_payload()
             .unwrap();
-        let pattern = crate::pattern::Pattern::from_bytes("MMM").unwrap();
-        let datetime = MockDateTime::try_new(2020, 8, 1, 12, 34, 28).unwrap();
+        let pattern = "MMM".parse().unwrap();
+        let datetime =
+            DateTime::new_gregorian_datetime_from_integers(2020, 8, 1, 12, 34, 28).unwrap();
         let mut sink = String::new();
-        write_pattern(
-            &pattern,
-            Some(&data.get()),
-            &datetime,
-            &"und".parse().unwrap(),
-            &mut sink,
-        )
-        .unwrap();
+        let loc_datetime = DateTimeInputWithLocale::new(&datetime, &"und".parse().unwrap());
+        write_pattern(&pattern, Some(data.get()), &loc_datetime, &mut sink).unwrap();
         println!("{}", sink);
     }
 
