@@ -6,18 +6,22 @@ use core::fmt;
 
 use crate::error::DateTimeFormatError as Error;
 use crate::fields::{self, FieldSymbol};
-use crate::pattern::{Error as PatternError, PatternItem};
+use crate::pattern::{PatternError, PatternItem};
+use crate::provider::calendar::patterns::PatternPluralsFromPatternsV1Marker;
 use crate::{
     date::TimeZoneInput,
-    time_zone::{IsoFormat, IsoMinutes, IsoSeconds, TimeZoneFormat},
+    time_zone::{
+        IsoFormat, IsoMinutes, IsoSeconds, TimeZoneFormat, TimeZoneFormatConfig, TimeZoneFormatKind,
+    },
 };
+use icu_provider::DataPayload;
 use writeable::Writeable;
 
 pub struct FormattedTimeZone<'l, T>
 where
     T: TimeZoneInput,
 {
-    pub(crate) time_zone_format: &'l TimeZoneFormat<'l>,
+    pub(crate) time_zone_format: &'l TimeZoneFormat,
     pub(crate) time_zone: &'l T,
 }
 
@@ -26,7 +30,7 @@ where
     T: TimeZoneInput,
 {
     fn write_to<W: fmt::Write + ?Sized>(&self, sink: &mut W) -> fmt::Result {
-        write_pattern(self.time_zone_format, self.time_zone, sink).map_err(|_| core::fmt::Error)
+        write_zone(self.time_zone_format, self.time_zone, sink).map_err(|_| core::fmt::Error)
     }
 
     // TODO(#489): Implement write_len
@@ -37,12 +41,11 @@ where
     T: TimeZoneInput,
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write_pattern(self.time_zone_format, self.time_zone, f).map_err(|_| core::fmt::Error)
+        write_zone(self.time_zone_format, self.time_zone, f).map_err(|_| core::fmt::Error)
     }
 }
 
-// TODO(#622) Make this public once TimeZoneFormat is public.
-pub(crate) fn write_pattern<T, W>(
+pub(crate) fn write_zone<T, W>(
     time_zone_format: &TimeZoneFormat,
     time_zone: &T,
     w: &mut W,
@@ -51,11 +54,81 @@ where
     T: TimeZoneInput,
     W: fmt::Write + ?Sized,
 {
-    let pattern = &time_zone_format.pattern;
-    for item in pattern.items() {
+    match &time_zone_format.kind {
+        TimeZoneFormatKind::Pattern(patterns) => {
+            write_pattern(time_zone_format, time_zone, patterns, w)
+        }
+        TimeZoneFormatKind::Config(config) => write_config(time_zone_format, time_zone, *config, w),
+    }
+}
+
+fn write_config<T, W>(
+    time_zone_format: &TimeZoneFormat,
+    time_zone: &T,
+    config: TimeZoneFormatConfig,
+    w: &mut W,
+) -> Result<(), Error>
+where
+    T: TimeZoneInput,
+    W: fmt::Write + ?Sized,
+{
+    match config {
+        TimeZoneFormatConfig::GenericNonLocationLong => {
+            time_zone_format
+                .long_generic_non_location_format(w, time_zone)
+                .or_else(|_| time_zone_format.generic_location_format(w, time_zone))
+                .or_else(|_| time_zone_format.localized_gmt_format(w, time_zone))?;
+        }
+        TimeZoneFormatConfig::GenericNonLocationShort => {
+            time_zone_format
+                .short_generic_non_location_format(w, time_zone)
+                .or_else(|_| time_zone_format.generic_location_format(w, time_zone))
+                .or_else(|_| time_zone_format.localized_gmt_format(w, time_zone))?;
+        }
+        TimeZoneFormatConfig::GenericLocation => {
+            time_zone_format
+                .generic_location_format(w, time_zone)
+                .or_else(|_| time_zone_format.localized_gmt_format(w, time_zone))?;
+        }
+        TimeZoneFormatConfig::SpecificNonLocationLong => {
+            time_zone_format
+                .long_specific_non_location_format(w, time_zone)
+                .or_else(|_| time_zone_format.localized_gmt_format(w, time_zone))?;
+        }
+        TimeZoneFormatConfig::SpecificNonLocationShort => {
+            time_zone_format
+                .short_specific_non_location_format(w, time_zone)
+                .or_else(|_| time_zone_format.localized_gmt_format(w, time_zone))?;
+        }
+        TimeZoneFormatConfig::LocalizedGMT => {
+            time_zone_format.localized_gmt_format(w, time_zone)?;
+        }
+        TimeZoneFormatConfig::Iso8601(iso_format, iso_minutes, iso_seconds) => {
+            time_zone_format.iso8601_format(w, time_zone, iso_format, iso_minutes, iso_seconds)?;
+        }
+    }
+    Ok(())
+}
+
+fn write_pattern<T, W>(
+    time_zone_format: &TimeZoneFormat,
+    time_zone: &T,
+    patterns: &DataPayload<PatternPluralsFromPatternsV1Marker>,
+    w: &mut W,
+) -> Result<(), Error>
+where
+    T: TimeZoneInput,
+    W: fmt::Write + ?Sized,
+{
+    let pattern = patterns
+        .get()
+        .0
+        .clone()
+        .expect_pattern("Expected a single pattern");
+    for item in pattern.items.iter() {
         match item {
             PatternItem::Field(field) => write_field(field, time_zone_format, time_zone, w)?,
-            PatternItem::Literal(l) => w.write_str(l)?,
+            PatternItem::Literal(ch) => w.write_char(ch)?,
         }
     }
     Ok(())
@@ -64,7 +137,7 @@ where
 /// Write fields according to the UTS-35 specification.
 /// https://unicode.org/reports/tr35/tr35-dates.html#dfst-zone
 pub(super) fn write_field<T, W>(
-    field: &fields::Field,
+    field: fields::Field,
     time_zone_format: &TimeZoneFormat,
     time_zone: &T,
     w: &mut W,
@@ -75,7 +148,7 @@ where
 {
     if let FieldSymbol::TimeZone(zone_symbol) = field.symbol {
         match zone_symbol {
-            fields::TimeZone::LowerZ => match u8::from(field.length) {
+            fields::TimeZone::LowerZ => match field.length.idx() {
                 1..=3 => time_zone_format
                     .short_specific_non_location_format(w, time_zone)
                     .or_else(|_| time_zone_format.localized_gmt_format(w, time_zone))?,
@@ -88,7 +161,7 @@ where
                     )))
                 }
             },
-            fields::TimeZone::UpperZ => match u8::from(field.length) {
+            fields::TimeZone::UpperZ => match field.length.idx() {
                 1..=3 => time_zone_format.iso8601_format(
                     w,
                     time_zone,
@@ -110,7 +183,7 @@ where
                     )))
                 }
             },
-            fields::TimeZone::UpperO => match u8::from(field.length) {
+            fields::TimeZone::UpperO => match field.length.idx() {
                 1..=4 => time_zone_format.localized_gmt_format(w, time_zone)?,
                 _ => {
                     return Err(Error::Pattern(PatternError::FieldLengthInvalid(
@@ -118,7 +191,7 @@ where
                     )))
                 }
             },
-            fields::TimeZone::LowerV => match u8::from(field.length) {
+            fields::TimeZone::LowerV => match field.length.idx() {
                 1 => time_zone_format
                     .short_generic_non_location_format(w, time_zone)
                     .or_else(|_| time_zone_format.generic_location_format(w, time_zone))
@@ -133,7 +206,7 @@ where
                     )))
                 }
             },
-            fields::TimeZone::UpperV => match u8::from(field.length) {
+            fields::TimeZone::UpperV => match field.length.idx() {
                 1 => todo!("#606 (BCP-47 identifiers)"),
                 2 => todo!("#606 (BCP-47 identifiers)"),
                 3 => time_zone_format
@@ -148,7 +221,7 @@ where
                     )))
                 }
             },
-            fields::TimeZone::LowerX => match u8::from(field.length) {
+            fields::TimeZone::LowerX => match field.length.idx() {
                 1 => time_zone_format.iso8601_format(
                     w,
                     time_zone,
@@ -190,7 +263,7 @@ where
                     )))
                 }
             },
-            fields::TimeZone::UpperX => match u8::from(field.length) {
+            fields::TimeZone::UpperX => match field.length.idx() {
                 1 => time_zone_format.iso8601_format(
                     w,
                     time_zone,
