@@ -2,26 +2,30 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
+//! A collection of code for formatting DateTimes with time zones.
+
+use crate::date::ZonedDateTimeInput;
 use crate::date::{LocalizedDateTimeInput, ZonedDateTimeInputWithLocale};
 use crate::error::DateTimeFormatError as Error;
 use crate::fields::{self, FieldSymbol};
-use crate::pattern::PatternItem;
-use crate::{date::ZonedDateTimeInput, zoned_datetime::ZonedDateTimeFormat};
+use crate::pattern::{runtime::Pattern, PatternItem};
+use crate::raw;
 use core::fmt;
 use writeable::Writeable;
 
 use super::datetime;
 use super::time_zone;
 
-pub struct FormattedZonedDateTime<'l, 'data, T>
+#[allow(missing_docs)] // TODO(#686) - Add missing docs.
+pub struct FormattedZonedDateTime<'l, T>
 where
     T: ZonedDateTimeInput,
 {
-    pub(crate) zoned_datetime_format: &'l ZonedDateTimeFormat<'data>,
+    pub(crate) zoned_datetime_format: &'l raw::ZonedDateTimeFormat,
     pub(crate) zoned_datetime: &'l T,
 }
 
-impl<'l, 'd, T> Writeable for FormattedZonedDateTime<'l, 'd, T>
+impl<'l, T> Writeable for FormattedZonedDateTime<'l, T>
 where
     T: ZonedDateTimeInput,
 {
@@ -33,7 +37,7 @@ where
     // TODO(#489): Implement write_len
 }
 
-impl<'l, 'd, T> fmt::Display for FormattedZonedDateTime<'l, 'd, T>
+impl<'l, T> fmt::Display for FormattedZonedDateTime<'l, T>
 where
     T: ZonedDateTimeInput,
 {
@@ -43,8 +47,8 @@ where
     }
 }
 
-pub fn write_pattern<T, W>(
-    zoned_datetime_format: &ZonedDateTimeFormat,
+pub(crate) fn write_pattern<T, W>(
+    zoned_datetime_format: &raw::ZonedDateTimeFormat,
     zoned_datetime: &T,
     w: &mut W,
 ) -> Result<(), Error>
@@ -53,22 +57,29 @@ where
     W: fmt::Write + ?Sized,
 {
     let locale = &zoned_datetime_format.datetime_format.locale;
-    let pattern = &zoned_datetime_format.datetime_format.pattern;
+    let patterns = &zoned_datetime_format.datetime_format.patterns;
     let loc_datetime = ZonedDateTimeInputWithLocale::new(zoned_datetime, locale);
-    for item in pattern.items() {
+
+    let pattern = patterns.get().0.select(
+        &loc_datetime,
+        zoned_datetime_format.datetime_format.ordinal_rules.as_ref(),
+    )?;
+
+    for item in pattern.items.iter() {
         match item {
             PatternItem::Field(field) => {
-                write_field(field, zoned_datetime_format, &loc_datetime, w)?
+                write_field(pattern, field, zoned_datetime_format, &loc_datetime, w)?
             }
-            PatternItem::Literal(l) => w.write_str(l)?,
+            PatternItem::Literal(ch) => w.write_char(ch)?,
         }
     }
     Ok(())
 }
 
 fn write_field<T, W>(
-    field: &fields::Field,
-    zoned_datetime_format: &ZonedDateTimeFormat,
+    pattern: &Pattern,
+    field: fields::Field,
+    zoned_datetime_format: &raw::ZonedDateTimeFormat,
     loc_datetime: &impl LocalizedDateTimeInput<T>,
     w: &mut W,
 ) -> Result<(), Error>
@@ -76,7 +87,6 @@ where
     T: ZonedDateTimeInput,
     W: fmt::Write + ?Sized,
 {
-    let pattern = &zoned_datetime_format.datetime_format.pattern;
     let symbols = zoned_datetime_format
         .datetime_format
         .symbols
