@@ -2,6 +2,8 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
+#![warn(missing_docs)]
+
 //! `icu_datetime` is one of the [`ICU4X`] components.
 //!
 //! This API provides necessary functionality for formatting date and time to user readable textual representation.
@@ -15,7 +17,8 @@
 //! ```
 //! use icu::locid::Locale;
 //! use icu::locid::macros::langid;
-//! use icu::datetime::{DateTimeFormat, DateTimeFormatOptions, mock::datetime::MockDateTime, options::length};
+//! use icu::calendar::Gregorian;
+//! use icu::datetime::{DateTimeFormat, DateTimeFormatOptions, mock::parse_gregorian_from_str, options::length};
 //!
 //! let provider = icu_testdata::get_provider();
 //!
@@ -28,11 +31,11 @@
 //!     ..Default::default()
 //! });
 //!
-//! let dtf = DateTimeFormat::try_new(locale, &provider, &options)
+//! let dtf = DateTimeFormat::<Gregorian>::try_new(locale, &provider, &options)
 //!     .expect("Failed to create DateTimeFormat instance.");
 //!
 //!
-//! let date: MockDateTime = "2020-09-12T12:35:00".parse()
+//! let date = parse_gregorian_from_str("2020-09-12T12:35:00")
 //!     .expect("Failed to parse date.");
 //!
 //! let formatted_date = dtf.format(&date);
@@ -45,7 +48,8 @@
 //! ```
 //! use icu::locid::Locale;
 //! use icu::locid::macros::langid;
-//! use icu::datetime::{DateTimeFormat, DateTimeFormatOptions, mock::datetime::MockDateTime, options::length};
+//! use icu::calendar::Gregorian;
+//! use icu::datetime::{DateTimeFormat, DateTimeFormatOptions, options::length};
 //! # let provider = icu_testdata::get_provider();
 //! # let locale: Locale = langid!("en").into();
 //! let options = length::Bag {
@@ -54,47 +58,53 @@
 //!     ..Default::default()
 //! }.into();
 //!
-//! let dtf = DateTimeFormat::try_new(locale, &provider, &options);
+//! let dtf = DateTimeFormat::<Gregorian>::try_new(locale, &provider, &options);
 //! ```
 //!
 //! At the moment, the crate provides only options using the [`Length`] bag, but in the future,
 //! we expect to add more ways to customize the output, like skeletons, and components.
 //!
-//! *Notice:* Rust at the moment does not have a canonical way to represent date and time. We are introducing
-//! [`MockDateTime`] as an example of the data necessary for ICU [`DateTimeFormat`] to work, and
+//! *Notice:* Rust at the moment does not have a canonical way to represent date and time. We use
+//! [`DateTime`] as an example of the data necessary for ICU [`DateTimeFormat`] to work, and
 //! [we hope to work with the community](https://github.com/unicode-org/icu4x/blob/main/docs/research/datetime.md)
-//! to develop core date and time APIs that will work as an input for this component.
+//! to develop core date and time APIs that will work as an input for this component. [`DateTime`] additionally
+//! has support for non-Gregorian calendars, which this module will eventually be able to format.
 //!
 //! [`DataProvider`]: icu_provider::DataProvider
 //! [`ICU4X`]: ../icu/index.html
 //! [`Length`]: options::length
-//! [`MockDateTime`]: mock::datetime::MockDateTime
+//! [`DateTime`]: icu_calendar::DateTime
 
 #![cfg_attr(not(any(test, feature = "std")), no_std)]
-
 extern crate alloc;
 
-mod arithmetic;
+mod calendar;
 pub mod date;
 pub mod datetime;
 mod error;
 mod fields;
+#[allow(missing_docs)] // TODO(#686) - Add missing docs.
 mod format;
 pub mod mock;
 pub mod options;
 #[doc(hidden)]
 pub mod pattern;
 pub mod provider;
+pub(crate) mod raw;
 #[doc(hidden)]
 pub mod skeleton;
-// TODO(#622) make the time_zone module public once TimeZoneFormat is public.
-pub(crate) mod time_zone;
+#[allow(missing_docs)] // TODO(#686) - Add missing docs.
+pub mod time_zone;
+#[allow(missing_docs)] // TODO(#686) - Add missing docs.
 pub mod zoned_datetime;
 
+pub use calendar::CldrCalendar;
 pub use datetime::DateTimeFormat;
 pub use error::DateTimeFormatError;
 pub use format::datetime::FormattedDateTime;
+pub use format::time_zone::FormattedTimeZone;
 pub use format::zoned_datetime::FormattedZonedDateTime;
 pub use options::DateTimeFormatOptions;
+pub use time_zone::TimeZoneFormat;
+pub use time_zone::TimeZoneFormatConfig;
 pub use zoned_datetime::ZonedDateTimeFormat;
-// TODO(#622) re-export TimeZoneFormat once it is made public.
