@@ -77,11 +77,14 @@ use serde::{Deserialize, Serialize};
 ///
 /// [`UTS #35: Unicode LDML 4. Dates`]: https://unicode.org/reports/tr35/tr35-dates.html
 /// [`Element dateFormats`]: https://unicode.org/reports/tr35/tr35-dates.html#dateFormats
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq, Copy)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct Bag {
+    /// Configure the date part of the datetime.
     pub date: Option<Date>,
+    /// Configure the time part of the datetime.
     pub time: Option<Time>,
+    /// Configure the preferences for the datetime, such as the hour cycle.
     pub preferences: Option<preferences::Bag>,
 }
 
