@@ -8,27 +8,30 @@ use alloc::string::String;
 use core::fmt;
 
 use crate::{
-    date::TimeZoneInput, format::time_zone::FormattedTimeZone, pattern::Error as PatternError,
-    provider, DateTimeFormatError,
+    date::TimeZoneInput,
+    error::DateTimeFormatError,
+    fields::{FieldSymbol, TimeZone},
+    format::time_zone::{self, FormattedTimeZone},
+    pattern::{PatternError, PatternItem},
+    provider::{
+        self, calendar::patterns::PatternPluralsFromPatternsV1Marker,
+        time_zones::TimeZoneFormatsV1Marker,
+    },
 };
-use crate::{format::time_zone, provider::time_zones::TimeZoneFormatsV1Marker};
 use icu_locid::{LanguageIdentifier, Locale};
 use icu_provider::prelude::*;
 
-use crate::fields::{FieldSymbol, TimeZone};
-use crate::pattern::{Pattern, PatternItem};
-
 /// Loads a resource into its destination if the destination has not already been filled.
-fn load_resource<'data, D, L, P>(
+fn load_resource<D, L, P>(
     locale: &L,
     resource_key: ResourceKey,
-    destination: &mut Option<DataPayload<'data, D>>,
+    destination: &mut Option<DataPayload<D>>,
     provider: &P,
 ) -> Result<(), DateTimeFormatError>
 where
-    D: DataMarker<'data>,
+    D: DataMarker,
     L: Clone + Into<LanguageIdentifier>,
-    P: DataProvider<'data, D> + ?Sized,
+    P: DataProvider<D> + ?Sized,
 {
     if destination.is_none() {
         *destination = Some(
@@ -49,11 +52,11 @@ where
 }
 
 /// [`TimeZoneFormat`] uses data from the [`DataProvider`], the selected [`Locale`], and the provided
-/// pattern to collect all data necessary to format time zones into that locale.
+/// [`TimeZoneFormatConfig`] to collect all data necessary to format time zones into that locale.
 ///
-/// The various time-zone pattern symbols specified in UTS-35 require different sets of data for
+/// The various time-zone configs specified in UTS-35 require different sets of data for
 /// formatting. As such,[`TimeZoneFormat`] will pull in only the resources needed to format the
-/// pattern that it is given upon construction.
+/// config that it is given upon construction.
 ///
 /// For that reason, one should think of the process of formatting a time zone in two steps:
 /// first, a computationally heavy construction of [`TimeZoneFormat`], and then fast formatting
@@ -61,90 +64,66 @@ where
 ///
 /// # Examples
 ///
-/// // TODO(#622) Uncomment and update example once TimeZoneFormat is public.
-/// // ```
-/// // use icu_locid::Locale;
-/// // use icu_locid::macros::langid;
-/// // use icu_datetime::TimeZoneFormat;
-/// // use icu_datetime::date::GmtOffset;
-/// // use icu_datetime::mock::time_zone::MockTimeZone;
-/// // use icu_provider::inv::InvariantDataProvider;
+/// ```
+/// use icu_locid::Locale;
+/// use icu::locid::macros::langid;
+/// use icu_datetime::{TimeZoneFormat, TimeZoneFormatConfig};
+/// use icu_datetime::date::GmtOffset;
+/// use icu_datetime::mock::time_zone::MockTimeZone;
+/// use icu_provider::inv::InvariantDataProvider;
 ///
-/// // let locale: Locale = langid!("en").into();
-/// // let pattern = std::iter::empty().collect();
-/// // let provider = InvariantDataProvider;
+/// let locale: Locale = langid!("en").into();
+/// let provider = InvariantDataProvider;
 ///
-/// // let tzf = TimeZoneFormat::try_new(locale, pattern, &provider)
-/// //     .expect("Failed to create TimeZoneFormat");
+/// let tzf = TimeZoneFormat::try_from_config(locale, TimeZoneFormatConfig::GenericNonLocationLong, &provider)
+///     .expect("Failed to create TimeZoneFormat");
 ///
-/// // let time_zone = MockTimeZone::new(
-/// //        GmtOffset::default(),
-/// //        None,
-/// //        None,
-/// //        None,
-/// // );
+/// let time_zone = MockTimeZone::new(
+///        GmtOffset::default(),
+///        None,
+///        None,
+///        None,
+/// );
 ///
-/// // let value = tzf.format_to_string(&time_zone);
-/// // ```
-// TODO(#622) Make TimeZoneFormat public once we have a clean way to provide it options.
-pub(super) struct TimeZoneFormat<'data> {
-    /// The pattern to format.
-    pub(super) pattern: Pattern,
+/// let value = tzf.format_to_string(&time_zone);
+/// ```
+pub struct TimeZoneFormat {
+    // The kind of time zone format.
+    pub(super) kind: TimeZoneFormatKind,
     /// The data that contains meta information about how to display content.
-    pub(super) zone_formats: DataPayload<'data, provider::time_zones::TimeZoneFormatsV1Marker>,
+    pub(super) zone_formats: DataPayload<provider::time_zones::TimeZoneFormatsV1Marker>,
     /// The exemplar cities for time zones.
-    pub(super) exemplar_cities:
-        Option<DataPayload<'data, provider::time_zones::ExemplarCitiesV1Marker>>,
+    pub(super) exemplar_cities: Option<DataPayload<provider::time_zones::ExemplarCitiesV1Marker>>,
     /// The generic long metazone names, e.g. Pacific Time
     pub(super) mz_generic_long:
-        Option<DataPayload<'data, provider::time_zones::MetaZoneGenericNamesLongV1Marker>>,
+        Option<DataPayload<provider::time_zones::MetaZoneGenericNamesLongV1Marker>>,
     /// The generic short metazone names, e.g. PT
     pub(super) mz_generic_short:
-        Option<DataPayload<'data, provider::time_zones::MetaZoneGenericNamesShortV1Marker>>,
+        Option<DataPayload<provider::time_zones::MetaZoneGenericNamesShortV1Marker>>,
     /// The specific long metazone names, e.g. Pacific Daylight Time
     pub(super) mz_specific_long:
-        Option<DataPayload<'data, provider::time_zones::MetaZoneSpecificNamesLongV1Marker>>,
+        Option<DataPayload<provider::time_zones::MetaZoneSpecificNamesLongV1Marker>>,
     /// The specific short metazone names, e.g. Pacific Daylight Time
     pub(super) mz_specific_short:
-        Option<DataPayload<'data, provider::time_zones::MetaZoneSpecificNamesShortV1Marker>>,
+        Option<DataPayload<provider::time_zones::MetaZoneSpecificNamesShortV1Marker>>,
 }
 
-impl<'data> TimeZoneFormat<'data> {
+impl TimeZoneFormat {
     /// Constructor that selectively loads data based on what is required to
     /// format the given pattern into the given locale.
-    ///
-    /// # Examples
-    ///
-    /// // TODO(#622) Uncomment and update example once TimeZoneFormat is public.
-    /// // ```
-    /// // use icu_locid::Locale;
-    /// // use icu_locid::macros::langid;
-    /// // use icu_datetime::TimeZoneFormat;
-    /// // use icu_datetime::mock::time_zone::MockTimeZone;
-    /// // use icu_provider::inv::InvariantDataProvider;
-    ///
-    /// // let locale: Locale = langid!("en").into();
-    /// // let pattern = std::iter::empty().collect();
-    /// // let provider = InvariantDataProvider;
-    ///
-    /// // let tzf = TimeZoneFormat::try_new(locale, pattern, &provider);
-    ///
-    /// // assert!(tzf.is_ok());
-    /// // ```
-    // TODO(#622) Make this public once TimeZoneFormat is public.
     pub(super) fn try_new<L, ZP>(
         locale: L,
-        pattern: Pattern,
+        patterns: DataPayload<PatternPluralsFromPatternsV1Marker>,
         zone_provider: &ZP,
     ) -> Result<Self, DateTimeFormatError>
     where
         L: Into<Locale>,
-        ZP: DataProvider<'data, provider::time_zones::TimeZoneFormatsV1Marker>
-            + DataProvider<'data, provider::time_zones::ExemplarCitiesV1Marker>
-            + DataProvider<'data, provider::time_zones::MetaZoneGenericNamesLongV1Marker>
-            + DataProvider<'data, provider::time_zones::MetaZoneGenericNamesShortV1Marker>
-            + DataProvider<'data, provider::time_zones::MetaZoneSpecificNamesLongV1Marker>
-            + DataProvider<'data, provider::time_zones::MetaZoneSpecificNamesShortV1Marker>
+        ZP: DataProvider<provider::time_zones::TimeZoneFormatsV1Marker>
+            + DataProvider<provider::time_zones::ExemplarCitiesV1Marker>
+            + DataProvider<provider::time_zones::MetaZoneGenericNamesLongV1Marker>
+            + DataProvider<provider::time_zones::MetaZoneGenericNamesShortV1Marker>
+            + DataProvider<provider::time_zones::MetaZoneSpecificNamesLongV1Marker>
+            + DataProvider<provider::time_zones::MetaZoneSpecificNamesShortV1Marker>
             + ?Sized,
     {
         let locale = locale.into();
@@ -161,42 +140,49 @@ impl<'data> TimeZoneFormat<'data> {
             })?
             .take_payload()?;
 
-        let mut time_zone_format = Self {
-            pattern,
-            zone_formats,
-            exemplar_cities: None,
-            mz_generic_long: None,
-            mz_generic_short: None,
-            mz_specific_long: None,
-            mz_specific_short: None,
-        };
-
-        let zone_symbols = time_zone_format
-            .pattern
-            .items()
-            .iter()
+        let zone_symbols = patterns
+            .get()
+            .0
+            .patterns_iter()
+            .map(|pattern| pattern.items.iter())
+            .flatten()
             .filter_map(|item| match item {
                 PatternItem::Field(field) => Some(field),
                 _ => None,
             })
             .filter_map(|field| match field.symbol {
-                FieldSymbol::TimeZone(zone) => Some((u8::from(field.length), zone)),
+                FieldSymbol::TimeZone(zone) => Some((field.length.idx(), zone)),
                 _ => None,
             });
 
+        let mut exemplar_cities: Option<DataPayload<provider::time_zones::ExemplarCitiesV1Marker>> =
+            None;
+        let mut mz_generic_long: Option<
+            DataPayload<provider::time_zones::MetaZoneGenericNamesLongV1Marker>,
+        > = None;
+        let mut mz_generic_short: Option<
+            DataPayload<provider::time_zones::MetaZoneGenericNamesShortV1Marker>,
+        > = None;
+        let mut mz_specific_long: Option<
+            DataPayload<provider::time_zones::MetaZoneSpecificNamesLongV1Marker>,
+        > = None;
+        let mut mz_specific_short: Option<
+            DataPayload<provider::time_zones::MetaZoneSpecificNamesShortV1Marker>,
+        > = None;
+
         for (length, symbol) in zone_symbols {
             match symbol {
                 TimeZone::LowerZ => match length {
                     1..=3 => load_resource(
                         &locale,
                         provider::key::TIMEZONE_SPECIFIC_NAMES_SHORT_V1,
-                        &mut time_zone_format.mz_specific_short,
+                        &mut mz_specific_short,
                         zone_provider,
                     )?,
                     4 => load_resource(
                         &locale,
                         provider::key::TIMEZONE_SPECIFIC_NAMES_LONG_V1,
-                        &mut time_zone_format.mz_specific_long,
+                        &mut mz_specific_long,
                         zone_provider,
                     )?,
                     _ => {
@@ -210,13 +196,13 @@ impl<'data> TimeZoneFormat<'data> {
                         load_resource(
                             &locale,
                             provider::key::TIMEZONE_GENERIC_NAMES_SHORT_V1,
-                            &mut time_zone_format.mz_generic_short,
+                            &mut mz_generic_short,
                             zone_provider,
                         )?;
                         load_resource(
                             &locale,
                             provider::key::TIMEZONE_EXEMPLAR_CITIES_V1,
-                            &mut time_zone_format.exemplar_cities,
+                            &mut exemplar_cities,
                             zone_provider,
                         )?;
                     }
@@ -224,13 +210,13 @@ impl<'data> TimeZoneFormat<'data> {
                         load_resource(
                             &locale,
                             provider::key::TIMEZONE_GENERIC_NAMES_LONG_V1,
-                            &mut time_zone_format.mz_generic_long,
+                            &mut mz_generic_long,
                             zone_provider,
                         )?;
                         load_resource(
                             &locale,
                             provider::key::TIMEZONE_EXEMPLAR_CITIES_V1,
-                            &mut time_zone_format.exemplar_cities,
+                            &mut exemplar_cities,
                             zone_provider,
                         )?;
                     }
@@ -246,7 +232,7 @@ impl<'data> TimeZoneFormat<'data> {
                     3 | 4 => load_resource(
                         &locale,
                         provider::key::TIMEZONE_EXEMPLAR_CITIES_V1,
-                        &mut time_zone_format.exemplar_cities,
+                        &mut exemplar_cities,
                         zone_provider,
                     )?,
                     _ => {
@@ -260,7 +246,145 @@ impl<'data> TimeZoneFormat<'data> {
             }
         }
 
-        Ok(time_zone_format)
+        Ok(Self {
+            kind: TimeZoneFormatKind::Pattern(patterns),
+            zone_formats,
+            exemplar_cities,
+            mz_generic_long,
+            mz_generic_short,
+            mz_specific_long,
+            mz_specific_short,
+        })
+    }
+
+    /// Constructor that selectively loads data based on what is required to
+    /// format the given config into the given locale.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu_locid::Locale;
+    /// use icu::locid::macros::langid;
+    /// use icu_datetime::{TimeZoneFormat, TimeZoneFormatConfig};
+    /// use icu_datetime::mock::time_zone::MockTimeZone;
+    /// use icu_provider::inv::InvariantDataProvider;
+    ///
+    /// let locale: Locale = langid!("en").into();
+    /// let provider = InvariantDataProvider;
+    ///
+    /// let tzf = TimeZoneFormat::try_from_config(locale, TimeZoneFormatConfig::LocalizedGMT, &provider);
+    ///
+    /// assert!(tzf.is_ok());
+    /// ```
+    pub fn try_from_config<L, ZP>(
+        locale: L,
+        config: TimeZoneFormatConfig,
+        zone_provider: &ZP,
+    ) -> Result<Self, DateTimeFormatError>
+    where
+        L: Into<Locale>,
+        ZP: DataProvider<provider::time_zones::TimeZoneFormatsV1Marker>
+            + DataProvider<provider::time_zones::ExemplarCitiesV1Marker>
+            + DataProvider<provider::time_zones::MetaZoneGenericNamesLongV1Marker>
+            + DataProvider<provider::time_zones::MetaZoneGenericNamesShortV1Marker>
+            + DataProvider<provider::time_zones::MetaZoneSpecificNamesLongV1Marker>
+            + DataProvider<provider::time_zones::MetaZoneSpecificNamesShortV1Marker>
+            + ?Sized,
+    {
+        let locale = locale.into();
+
+        let zone_formats: DataPayload<TimeZoneFormatsV1Marker> = zone_provider
+            .load_payload(&DataRequest {
+                resource_path: ResourcePath {
+                    key: provider::key::TIMEZONE_FORMATS_V1,
+                    options: ResourceOptions {
+                        variant: None,
+                        langid: Some(locale.clone().into()),
+                    },
+                },
+            })?
+            .take_payload()?;
+
+        let mut exemplar_cities: Option<DataPayload<provider::time_zones::ExemplarCitiesV1Marker>> =
+            None;
+        let mut mz_generic_long: Option<
+            DataPayload<provider::time_zones::MetaZoneGenericNamesLongV1Marker>,
+        > = None;
+        let mut mz_generic_short: Option<
+            DataPayload<provider::time_zones::MetaZoneGenericNamesShortV1Marker>,
+        > = None;
+        let mut mz_specific_long: Option<
+            DataPayload<provider::time_zones::MetaZoneSpecificNamesLongV1Marker>,
+        > = None;
+        let mut mz_specific_short: Option<
+            DataPayload<provider::time_zones::MetaZoneSpecificNamesShortV1Marker>,
+        > = None;
+
+        match config {
+            TimeZoneFormatConfig::GenericNonLocationLong => {
+                load_resource(
+                    &locale,
+                    provider::key::TIMEZONE_GENERIC_NAMES_LONG_V1,
+                    &mut mz_generic_long,
+                    zone_provider,
+                )?;
+                load_resource(
+                    &locale,
+                    provider::key::TIMEZONE_EXEMPLAR_CITIES_V1,
+                    &mut exemplar_cities,
+                    zone_provider,
+                )?;
+            }
+            TimeZoneFormatConfig::GenericNonLocationShort => {
+                load_resource(
+                    &locale,
+                    provider::key::TIMEZONE_GENERIC_NAMES_SHORT_V1,
+                    &mut mz_generic_short,
+                    zone_provider,
+                )?;
+                load_resource(
+                    &locale,
+                    provider::key::TIMEZONE_EXEMPLAR_CITIES_V1,
+                    &mut exemplar_cities,
+                    zone_provider,
+                )?;
+            }
+            TimeZoneFormatConfig::GenericLocation => {
+                load_resource(
+                    &locale,
+                    provider::key::TIMEZONE_EXEMPLAR_CITIES_V1,
+                    &mut exemplar_cities,
+                    zone_provider,
+                )?;
+            }
+            TimeZoneFormatConfig::SpecificNonLocationLong => {
+                load_resource(
+                    &locale,
+                    provider::key::TIMEZONE_SPECIFIC_NAMES_LONG_V1,
+                    &mut mz_specific_long,
+                    zone_provider,
+                )?;
+            }
+            TimeZoneFormatConfig::SpecificNonLocationShort => {
+                load_resource(
+                    &locale,
+                    provider::key::TIMEZONE_SPECIFIC_NAMES_SHORT_V1,
+                    &mut mz_specific_short,
+                    zone_provider,
+                )?;
+            }
+            TimeZoneFormatConfig::LocalizedGMT | TimeZoneFormatConfig::Iso8601(..) => (),
+        }
+
+        Ok(Self {
+            kind: TimeZoneFormatKind::Config(config),
+            zone_formats,
+            exemplar_cities,
+            mz_generic_long,
+            mz_generic_short,
+            mz_specific_long,
+            mz_specific_short,
+        })
     }
 
     /// Takes a [`TimeZoneInput`] implementer and returns an instance of a [`FormattedTimeZone`]
@@ -268,35 +392,30 @@ impl<'data> TimeZoneFormat<'data> {
     ///
     /// # Examples
     ///
-    /// // TODO(#622) Uncomment and update example once TimeZoneFormat is public.
-    /// // ```
-    /// // use icu_locid::Locale;
-    /// // use icu_locid::macros::langid;
-    /// // use icu_datetime::TimeZoneFormat;
-    /// // use icu_datetime::date::GmtOffset;
-    /// // use icu_datetime::mock::time_zone::MockTimeZone;
-    /// // use icu_provider::inv::InvariantDataProvider;
+    /// ```
+    /// use icu_locid::Locale;
+    /// use icu::locid::macros::langid;
+    /// use icu_datetime::{TimeZoneFormat, TimeZoneFormatConfig};
+    /// use icu_datetime::date::GmtOffset;
+    /// use icu_datetime::mock::time_zone::MockTimeZone;
+    /// use icu_provider::inv::InvariantDataProvider;
     ///
-    /// // # let locale: Locale = langid!("en").into();
-    /// // # let pattern = std::iter::empty().collect();
-    /// // # let provider = InvariantDataProvider;
+    /// let locale: Locale = langid!("en").into();
+    /// let provider = InvariantDataProvider;
     ///
-    /// // let tzf = TimeZoneFormat::try_new(locale, pattern, &provider)
-    /// //     .expect("Failed to create TimeZoneFormat");
+    /// let tzf = TimeZoneFormat::try_from_config(locale, TimeZoneFormatConfig::LocalizedGMT, &provider)
+    ///     .expect("Failed to create TimeZoneFormat");
     ///
-    /// // let time_zone = MockTimeZone::new(
-    /// //        GmtOffset::default(),
-    /// //        None,
-    /// //        None,
-    /// //        None,
-    /// // );
+    /// let time_zone = MockTimeZone::new(
+    ///        GmtOffset::default(),
+    ///        None,
+    ///        None,
+    ///        None,
+    /// );
     ///
-    /// // let _ = tzf.format(&time_zone);
-    /// // ```
-    // TODO(#622) Make this public once TimeZoneFormat is public.
-    //           And remove #[allow(unused)]
-    #[allow(unused)]
-    pub(super) fn format<'l: 'data, T>(&'l self, value: &'l T) -> FormattedTimeZone<'l, T>
+    /// let _ = tzf.format(&time_zone);
+    /// ```
+    pub fn format<'l, T>(&'l self, value: &'l T) -> FormattedTimeZone<'l, T>
     where
         T: TimeZoneInput,
     {
@@ -311,79 +430,69 @@ impl<'data> TimeZoneFormat<'data> {
     ///
     /// # Examples
     ///
-    /// // TODO(#622) Uncomment and update example once TimeZoneFormat is public.
-    /// // ```
-    /// // use icu_locid::Locale;
-    /// // use icu_locid::macros::langid;
-    /// // use icu_datetime::TimeZoneFormat;
-    /// // use icu_datetime::date::GmtOffset;
-    /// // use icu_datetime::mock::time_zone::MockTimeZone;
-    /// // use icu_provider::inv::InvariantDataProvider;
+    /// ```
+    /// use icu_locid::Locale;
+    /// use icu::locid::macros::langid;
+    /// use icu_datetime::{TimeZoneFormat, TimeZoneFormatConfig};
+    /// use icu_datetime::date::GmtOffset;
+    /// use icu_datetime::mock::time_zone::MockTimeZone;
+    /// use icu_provider::inv::InvariantDataProvider;
     ///
-    /// // # let locale: Locale = langid!("en").into();
-    /// // # let pattern = std::iter::empty().collect();
-    /// // # let provider = InvariantDataProvider;
+    /// let locale: Locale = langid!("en").into();
+    /// let provider = InvariantDataProvider;
     ///
-    /// // let tzf = TimeZoneFormat::try_new(locale, pattern, &provider)
-    /// //     .expect("Failed to create TimeZoneFormat");
+    /// let tzf = TimeZoneFormat::try_from_config(locale, TimeZoneFormatConfig::LocalizedGMT, &provider)
+    ///     .expect("Failed to create TimeZoneFormat");
     ///
-    /// // let time_zone = MockTimeZone::new(
-    /// //        GmtOffset::default(),
-    /// //        None,
-    /// //        None,
-    /// //        None,
-    /// // );
+    /// let time_zone = MockTimeZone::new(
+    ///        GmtOffset::default(),
+    ///        None,
+    ///        None,
+    ///        None,
+    /// );
     ///
-    /// // let mut buffer = String::new();
-    /// // tzf.format_to_write(&mut buffer, &time_zone)
-    /// //     .expect("Failed to write to a buffer.");
+    /// let mut buffer = String::new();
+    /// tzf.format_to_write(&mut buffer, &time_zone)
+    ///     .expect("Failed to write to a buffer.");
     ///
-    /// // let _ = format!("Time Zone: {}", buffer);
-    /// // ```
-    // TODO(#622) Make this public once TimeZoneFormat is public.
-    //            And remove #[allow(unused)]
-    #[allow(unused)]
-    pub(super) fn format_to_write(
+    /// let _ = format!("Time Zone: {}", buffer);
+    /// ```
+    pub fn format_to_write(
         &self,
         w: &mut impl core::fmt::Write,
         value: &impl TimeZoneInput,
     ) -> fmt::Result {
-        time_zone::write_pattern(self, value, w).map_err(|_| core::fmt::Error)
+        time_zone::write_zone(self, value, w).map_err(|_| core::fmt::Error)
     }
 
     /// Takes a [`TimeZoneInput`] implementer and returns a string with the formatted value.
     ///
     /// # Examples
     ///
-    /// // TODO(#622) Uncomment and update example once TimeZoneFormat is public.
-    /// // ```
-    /// // use icu_locid::Locale;
-    /// // use icu_locid::macros::langid;
-    /// // use icu_datetime::TimeZoneFormat;
-    /// // use icu_datetime::date::GmtOffset;
-    /// // use icu_datetime::mock::time_zone::MockTimeZone;
-    /// // use icu_provider::inv::InvariantDataProvider;
+    /// ```
+    /// use icu_locid::Locale;
+    /// use icu::locid::macros::langid;
+    /// use icu_datetime::{TimeZoneFormat, TimeZoneFormatConfig};
+    /// use icu_datetime::date::GmtOffset;
+    /// use icu_datetime::mock::time_zone::MockTimeZone;
+    /// use icu_provider::inv::InvariantDataProvider;
     ///
-    /// // # let locale: Locale = langid!("en").into();
-    /// // # let pattern = std::iter::empty().collect();
-    /// // # let provider = InvariantDataProvider;
+    /// let locale: Locale = langid!("en").into();
+    /// let provider = InvariantDataProvider;
     ///
-    /// // let tzf = TimeZoneFormat::try_new(locale, pattern, &provider)
-    /// //     .expect("Failed to create TimeZoneFormat");
+    /// let tzf = TimeZoneFormat::try_from_config(locale, TimeZoneFormatConfig::LocalizedGMT, &provider)
+    ///     .expect("Failed to create TimeZoneFormat");
     ///
-    /// // let time_zone = MockTimeZone::new(
-    /// //        GmtOffset::default(),
-    /// //        None,
-    /// //        None,
-    /// //        None,
-    /// // );
+    /// let time_zone = MockTimeZone::new(
+    ///        GmtOffset::default(),
+    ///        None,
+    ///        None,
+    ///        None,
+    /// );
     ///
-    /// // let _ = tzf.format_to_string(&time_zone);
-    /// // ```
-    // TODO(#622) Make this public once TimeZoneFormat is public.
-    //           And remove #[allow(unused)]
-    #[allow(unused)]
-    pub(super) fn format_to_string(&self, value: &impl TimeZoneInput) -> String {
+    /// let _ = tzf.format_to_string(&time_zone);
+    /// ```
+    pub fn format_to_string(&self, value: &impl TimeZoneInput) -> String {
         let mut s = String::new();
         self.format_to_write(&mut s, value)
             .expect("Failed to write to a String.");
@@ -428,7 +537,19 @@ impl<'data> TimeZoneFormat<'data> {
             self.mz_generic_short
                 .as_ref()
                 .map(|p| p.get())
-                .and_then(|metazones| time_zone.metazone_id().and_then(|mz| metazones.get(mz)))
+                .and_then(|metazones| {
+                    time_zone
+                        .time_zone_id()
+                        .and_then(|tz| metazones.get_override(tz))
+                })
+                .or_else(|| {
+                    self.mz_generic_short
+                        .as_ref()
+                        .map(|p| p.get())
+                        .and_then(|metazones| {
+                            time_zone.metazone_id().and_then(|mz| metazones.get(mz))
+                        })
+                })
                 .ok_or(fmt::Error)?,
         )
         .map_err(DateTimeFormatError::from)
@@ -446,7 +567,19 @@ impl<'data> TimeZoneFormat<'data> {
             self.mz_generic_long
                 .as_ref()
                 .map(|p| p.get())
-                .and_then(|metazones| time_zone.metazone_id().and_then(|mz| metazones.get(mz)))
+                .and_then(|metazones| {
+                    time_zone
+                        .time_zone_id()
+                        .and_then(|tz| metazones.get_override(tz))
+                })
+                .or_else(|| {
+                    self.mz_generic_long
+                        .as_ref()
+                        .map(|p| p.get())
+                        .and_then(|metazones| {
+                            time_zone.metazone_id().and_then(|mz| metazones.get(mz))
+                        })
+                })
                 .ok_or(fmt::Error)?,
         )
         .map_err(DateTimeFormatError::from)
@@ -464,11 +597,28 @@ impl<'data> TimeZoneFormat<'data> {
             self.mz_specific_short
                 .as_ref()
                 .map(|p| p.get())
-                .and_then(|metazones| time_zone.metazone_id().and_then(|mz| metazones.get(mz)))
-                .and_then(|specific_names| {
+                .and_then(|metazones| {
                     time_zone
-                        .time_variant()
-                        .and_then(|variant| specific_names.get(variant))
+                        .time_zone_id()
+                        .and_then(|tz| metazones.get_override(tz))
+                        .and_then(|specific_names| {
+                            time_zone
+                                .time_variant()
+                                .and_then(|variant| specific_names.get(variant))
+                        })
+                })
+                .or_else(|| {
+                    self.mz_specific_short
+                        .as_ref()
+                        .map(|p| p.get())
+                        .and_then(|metazones| {
+                            time_zone.metazone_id().and_then(|mz| metazones.get(mz))
+                        })
+                        .and_then(|specific_names| {
+                            time_zone
+                                .time_variant()
+                                .and_then(|variant| specific_names.get(variant))
+                        })
                 })
                 .ok_or_else(|| DateTimeFormatError::from(fmt::Error))?,
         )
@@ -487,11 +637,28 @@ impl<'data> TimeZoneFormat<'data> {
             self.mz_specific_long
                 .as_ref()
                 .map(|p| p.get())
-                .and_then(|metazones| time_zone.metazone_id().and_then(|mz| metazones.get(mz)))
-                .and_then(|specific_names| {
+                .and_then(|metazones| {
                     time_zone
-                        .time_variant()
-                        .and_then(|variant| specific_names.get(variant))
+                        .time_zone_id()
+                        .and_then(|tz| metazones.get_override(tz))
+                        .and_then(|specific_names| {
+                            time_zone
+                                .time_variant()
+                                .and_then(|variant| specific_names.get(variant))
+                        })
+                })
+                .or_else(|| {
+                    self.mz_specific_long
+                        .as_ref()
+                        .map(|p| p.get())
+                        .and_then(|metazones| {
+                            time_zone.metazone_id().and_then(|mz| metazones.get(mz))
+                        })
+                        .and_then(|specific_names| {
+                            time_zone
+                                .time_variant()
+                                .and_then(|variant| specific_names.get(variant))
+                        })
                 })
                 .ok_or(fmt::Error)?,
         )
@@ -669,8 +836,16 @@ impl<'data> TimeZoneFormat<'data> {
     }
 }
 
+/// Determines which type of formats time zone uses. It can be either config or pattern.
+#[allow(clippy::large_enum_variant)]
+pub(super) enum TimeZoneFormatKind {
+    Config(TimeZoneFormatConfig),
+    Pattern(DataPayload<PatternPluralsFromPatternsV1Marker>),
+}
+
 /// Determines which ISO-8601 format should be used to format a [`GmtOffset`](crate::date::GmtOffset).
-pub(super) enum IsoFormat {
+#[derive(Debug, Clone, Copy, PartialEq)]
+pub enum IsoFormat {
     /// ISO-8601 Basic Format.
     /// Formats zero-offset numerically.
     /// e.g. +0500, +0000
@@ -693,7 +868,8 @@ pub(super) enum IsoFormat {
 }
 
 /// Whether the minutes field should be optional or required in ISO-8601 format.
-pub(super) enum IsoMinutes {
+#[derive(Debug, Clone, Copy, PartialEq)]
+pub enum IsoMinutes {
     /// Minutes are always displayed.
     Required,
 
@@ -702,7 +878,8 @@ pub(super) enum IsoMinutes {
 }
 
 /// Whether the seconds field should be optional or excluded in ISO-8601 format.
-pub(super) enum IsoSeconds {
+#[derive(Debug, Clone, Copy, PartialEq)]
+pub enum IsoSeconds {
     /// Seconds are displayed only if they are non-zero.
     Optional,
 
@@ -711,10 +888,23 @@ pub(super) enum IsoSeconds {
 }
 
 /// Whether a field should be zero-padded in ISO-8601 format.
-pub(super) enum ZeroPadding {
+#[derive(Debug, Clone, Copy, PartialEq)]
+pub enum ZeroPadding {
     /// Add zero-padding.
     On,
 
     /// Do not add zero-padding.
     Off,
 }
+
+/// A config enum for initializing TimeZoneFormat.
+#[derive(Debug, Clone, Copy, PartialEq)]
+pub enum TimeZoneFormatConfig {
+    GenericNonLocationLong,                     // Pacific Time
+    GenericNonLocationShort,                    // PT
+    GenericLocation,                            // Los Angeles Time
+    SpecificNonLocationLong,                    // Pacific Standard Time
+    SpecificNonLocationShort,                   // PDT
+    LocalizedGMT,                               // GMT-07:00
+    Iso8601(IsoFormat, IsoMinutes, IsoSeconds), // -07:00
+}
