@@ -3,24 +3,23 @@
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
 use alloc::string::String;
+use core::marker::PhantomData;
 use icu_locid::Locale;
-use icu_provider::{DataProvider, DataRequest, ResourceOptions, ResourcePath};
+use icu_plurals::provider::PluralRulesV1Marker;
+use icu_provider::DataProvider;
 
 use crate::{
     date::ZonedDateTimeInput,
-    datetime::DateTimeFormat,
-    format::{
-        datetime,
-        zoned_datetime::{self, FormattedZonedDateTime},
-    },
+    format::zoned_datetime::FormattedZonedDateTime,
     options::DateTimeFormatOptions,
-    provider::{self, helpers::DateTimePatterns},
-    time_zone::TimeZoneFormat,
-    DateTimeFormatError,
+    provider::{
+        self,
+        calendar::{DatePatternsV1Marker, DateSkeletonPatternsV1Marker, DateSymbolsV1Marker},
+    },
+    raw, CldrCalendar, DateTimeFormatError,
 };
 
-// TODO(#622) link [`TimeZoneFormat`] once it is public.
-/// The composition of [`DateTimeFormat`] and `TimeZoneFormat`.
+/// The composition of [`DateTimeFormat`](crate::DateTimeFormat) and [`TimeZoneFormat`](crate::TimeZoneFormat).
 ///
 /// [`ZonedDateTimeFormat`] uses data from the [`DataProvider`]s, the selected [`Locale`], and the
 /// provided pattern to collect all data necessary to format a datetime with time zones into that locale.
@@ -38,6 +37,7 @@ use crate::{
 /// ```
 /// use icu::locid::Locale;
 /// use icu::locid::macros::langid;
+/// use icu::calendar::Gregorian;
 /// use icu::datetime::{ZonedDateTimeFormat, options::length};
 /// use icu::datetime::mock::zoned_datetime::MockZonedDateTime;
 /// use icu_provider::inv::InvariantDataProvider;
@@ -46,13 +46,14 @@ use crate::{
 ///
 /// let date_provider = InvariantDataProvider;
 /// let zone_provider = InvariantDataProvider;
+/// let plural_provider = InvariantDataProvider;
 ///
 /// let options = length::Bag {
 ///     date: Some(length::Date::Medium),
 ///     time: Some(length::Time::Short),
 ///     ..Default::default()
 /// };
-/// let zdtf = ZonedDateTimeFormat::try_new(locale, &date_provider, &zone_provider, &options.into())
+/// let zdtf = ZonedDateTimeFormat::<Gregorian>::try_new(locale, &date_provider, &zone_provider, &plural_provider, &options.into())
 ///     .expect("Failed to create DateTimeFormat instance.");
 ///
 ///
@@ -62,12 +63,9 @@ use crate::{
 ///
 /// let value = zdtf.format_to_string(&zoned_datetime);
 /// ```
-pub struct ZonedDateTimeFormat<'data> {
-    pub(super) datetime_format: DateTimeFormat<'data>,
-    pub(super) time_zone_format: TimeZoneFormat<'data>,
-}
+pub struct ZonedDateTimeFormat<C>(raw::ZonedDateTimeFormat, PhantomData<C>);
 
-impl<'data> ZonedDateTimeFormat<'data> {
+impl<C: CldrCalendar> ZonedDateTimeFormat<C> {
     /// Constructor that takes a selected [`Locale`], a reference to a [`DataProvider`] for
     /// dates, a [`DataProvider`] for time zones, and a list of [`DateTimeFormatOptions`].
     /// It collects all data necessary to format zoned datetime values into the given locale.
@@ -77,6 +75,7 @@ impl<'data> ZonedDateTimeFormat<'data> {
     /// ```
     /// use icu::locid::Locale;
     /// use icu::locid::macros::langid;
+    /// use icu::calendar::Gregorian;
     /// use icu::datetime::{ZonedDateTimeFormat, DateTimeFormatOptions};
     /// use icu::datetime::mock::zoned_datetime::MockZonedDateTime;
     /// use icu_provider::inv::InvariantDataProvider;
@@ -85,83 +84,47 @@ impl<'data> ZonedDateTimeFormat<'data> {
     ///
     /// let date_provider = InvariantDataProvider;
     /// let zone_provider = InvariantDataProvider;
+    /// let plural_provider = InvariantDataProvider;
     ///
     /// let options = DateTimeFormatOptions::default();
     ///
-    /// let zdtf = ZonedDateTimeFormat::try_new(locale, &date_provider, &zone_provider, &options);
+    /// let zdtf = ZonedDateTimeFormat::<Gregorian>::try_new(locale, &date_provider, &zone_provider, &plural_provider, &options);
     ///
     /// assert_eq!(zdtf.is_ok(), true);
     /// ```
-    pub fn try_new<L, DP, ZP>(
+    #[inline]
+    pub fn try_new<L, DP, ZP, PP>(
         locale: L,
         date_provider: &DP,
         zone_provider: &ZP,
+        plural_provider: &PP,
         options: &DateTimeFormatOptions,
     ) -> Result<Self, DateTimeFormatError>
     where
         L: Into<Locale>,
-        DP: DataProvider<'data, provider::gregory::DatePatternsV1Marker>
-            + DataProvider<'data, provider::gregory::DateSymbolsV1Marker>
-            + ?Sized,
-        ZP: DataProvider<'data, provider::time_zones::TimeZoneFormatsV1Marker>
-            + DataProvider<'data, provider::time_zones::ExemplarCitiesV1Marker>
-            + DataProvider<'data, provider::time_zones::MetaZoneGenericNamesLongV1Marker>
-            + DataProvider<'data, provider::time_zones::MetaZoneGenericNamesShortV1Marker>
-            + DataProvider<'data, provider::time_zones::MetaZoneSpecificNamesLongV1Marker>
-            + DataProvider<'data, provider::time_zones::MetaZoneSpecificNamesShortV1Marker>
+        DP: DataProvider<DateSymbolsV1Marker>
+            + DataProvider<DatePatternsV1Marker>
+            + DataProvider<DateSkeletonPatternsV1Marker>,
+        ZP: DataProvider<provider::time_zones::TimeZoneFormatsV1Marker>
+            + DataProvider<provider::time_zones::ExemplarCitiesV1Marker>
+            + DataProvider<provider::time_zones::MetaZoneGenericNamesLongV1Marker>
+            + DataProvider<provider::time_zones::MetaZoneGenericNamesShortV1Marker>
+            + DataProvider<provider::time_zones::MetaZoneSpecificNamesLongV1Marker>
+            + DataProvider<provider::time_zones::MetaZoneSpecificNamesShortV1Marker>
             + ?Sized,
+        PP: DataProvider<PluralRulesV1Marker>,
     {
-        let locale = locale.into();
-        let pattern_data: icu_provider::DataPayload<'_, provider::gregory::DatePatternsV1Marker> =
-            date_provider
-                .load_payload(&DataRequest {
-                    resource_path: ResourcePath {
-                        key: provider::key::GREGORY_DATE_PATTERNS_V1,
-                        options: ResourceOptions {
-                            variant: None,
-                            langid: Some(locale.clone().into()),
-                        },
-                    },
-                })?
-                .take_payload()?;
-
-        let pattern = pattern_data
-            .get()
-            .get_pattern_for_options(options)?
-            .unwrap_or_default();
-
-        let requires_data = datetime::analyze_pattern(&pattern, true)
-            .map_err(|field| DateTimeFormatError::UnsupportedField(field.symbol))?;
-
-        let symbols_data = if requires_data {
-            Some(
-                date_provider
-                    .load_payload(&DataRequest {
-                        resource_path: ResourcePath {
-                            key: provider::key::GREGORY_DATE_SYMBOLS_V1,
-                            options: ResourceOptions {
-                                variant: None,
-                                langid: Some(locale.clone().into()),
-                            },
-                        },
-                    })?
-                    .take_payload()?,
-            )
-        } else {
-            None
-        };
-
-        let datetime_format = DateTimeFormat::new(locale, pattern, symbols_data);
-        let time_zone_format = TimeZoneFormat::try_new(
-            datetime_format.locale.clone(),
-            datetime_format.pattern.clone(),
-            zone_provider,
-        )?;
-
-        Ok(Self {
-            datetime_format,
-            time_zone_format,
-        })
+        Ok(Self(
+            raw::ZonedDateTimeFormat::try_new(
+                locale,
+                date_provider,
+                zone_provider,
+                plural_provider,
+                options,
+                C::IDENTIFIER,
+            )?,
+            PhantomData,
+        ))
     }
 
     /// Takes a [`ZonedDateTimeInput`] implementer and returns an instance of a [`FormattedZonedDateTime`]
@@ -172,14 +135,16 @@ impl<'data> ZonedDateTimeFormat<'data> {
     /// ```
     /// use icu::locid::Locale;
     /// use icu::locid::macros::langid;
+    /// use icu::calendar::Gregorian;
     /// use icu::datetime::{ZonedDateTimeFormat, DateTimeFormatOptions};
     /// use icu::datetime::mock::zoned_datetime::MockZonedDateTime;
     /// use icu_provider::inv::InvariantDataProvider;
     /// # let locale: Locale = langid!("en").into();
     /// # let date_provider = InvariantDataProvider;
     /// # let zone_provider = InvariantDataProvider;
+    /// # let plural_provider = InvariantDataProvider;
     /// # let options = DateTimeFormatOptions::default();
-    /// let zdtf = ZonedDateTimeFormat::try_new(locale, &date_provider, &zone_provider, &options)
+    /// let zdtf = ZonedDateTimeFormat::<Gregorian>::try_new(locale, &date_provider, &zone_provider, &plural_provider, &options)
     ///     .expect("Failed to create ZonedDateTimeFormat instance.");
     ///
     /// let zoned_datetime: MockZonedDateTime = "2021-04-08T16:12:37.000-07:00"
@@ -194,14 +159,12 @@ impl<'data> ZonedDateTimeFormat<'data> {
     /// At the moment, there's little value in using that over one of the other `format` methods,
     /// but [`FormattedZonedDateTime`] will grow with methods for iterating over fields, extracting information
     /// about formatted date and so on.
-    pub fn format<'l, T>(&'l self, value: &'l T) -> FormattedZonedDateTime<'l, 'data, T>
+    #[inline]
+    pub fn format<'l, T>(&'l self, value: &'l T) -> FormattedZonedDateTime<'l, T>
     where
         T: ZonedDateTimeInput,
     {
-        FormattedZonedDateTime {
-            zoned_datetime_format: self,
-            zoned_datetime: value,
-        }
+        self.0.format(value)
     }
 
     /// Takes a mutable reference to anything that implements the [`Write`](std::fmt::Write) trait
@@ -212,14 +175,16 @@ impl<'data> ZonedDateTimeFormat<'data> {
     /// ```
     /// use icu::locid::Locale;
     /// use icu::locid::macros::langid;
+    /// use icu::calendar::Gregorian;
     /// use icu::datetime::{ZonedDateTimeFormat, DateTimeFormatOptions};
     /// use icu::datetime::mock::zoned_datetime::MockZonedDateTime;
     /// use icu_provider::inv::InvariantDataProvider;
     /// # let locale: Locale = langid!("en").into();
     /// # let date_provider = InvariantDataProvider;
     /// # let zone_provider = InvariantDataProvider;
+    /// # let plural_provider = InvariantDataProvider;
     /// # let options = DateTimeFormatOptions::default();
-    /// let zdtf = ZonedDateTimeFormat::try_new(locale, &date_provider, &zone_provider, &options.into())
+    /// let zdtf = ZonedDateTimeFormat::<Gregorian>::try_new(locale, &date_provider, &zone_provider, &plural_provider, &options.into())
     ///     .expect("Failed to create ZonedDateTimeFormat instance.");
     ///
     /// let zoned_datetime: MockZonedDateTime = "2021-04-08T16:12:37.000-07:00"
@@ -232,12 +197,13 @@ impl<'data> ZonedDateTimeFormat<'data> {
     ///
     /// let _ = format!("Date: {}", buffer);
     /// ```
+    #[inline]
     pub fn format_to_write(
         &self,
         w: &mut impl core::fmt::Write,
         value: &impl ZonedDateTimeInput,
     ) -> core::fmt::Result {
-        zoned_datetime::write_pattern(self, value, w).map_err(|_| core::fmt::Error)
+        self.0.format_to_write(w, value)
     }
 
     /// Takes a [`ZonedDateTimeInput`] implementer and returns it formatted as a string.
@@ -247,14 +213,16 @@ impl<'data> ZonedDateTimeFormat<'data> {
     /// ```
     /// use icu::locid::Locale;
     /// use icu::locid::macros::langid;
+    /// use icu::calendar::Gregorian;
     /// use icu::datetime::{ZonedDateTimeFormat, DateTimeFormatOptions};
     /// use icu::datetime::mock::zoned_datetime::MockZonedDateTime;
     /// use icu_provider::inv::InvariantDataProvider;
     /// # let locale: Locale = langid!("en").into();
     /// # let date_provider = InvariantDataProvider;
     /// # let zone_provider = InvariantDataProvider;
+    /// # let plural_provider = InvariantDataProvider;
     /// # let options = DateTimeFormatOptions::default();
-    /// let zdtf = ZonedDateTimeFormat::try_new(locale, &date_provider, &zone_provider, &options.into())
+    /// let zdtf = ZonedDateTimeFormat::<Gregorian>::try_new(locale, &date_provider, &zone_provider, &plural_provider, &options.into())
     ///     .expect("Failed to create ZonedDateTimeFormat instance.");
     ///
     /// let zoned_datetime: MockZonedDateTime = "2021-04-08T16:12:37.000-07:00"
@@ -263,10 +231,8 @@ impl<'data> ZonedDateTimeFormat<'data> {
     ///
     /// let _ = zdtf.format_to_string(&zoned_datetime);
     /// ```
+    #[inline]
     pub fn format_to_string(&self, value: &impl ZonedDateTimeInput) -> String {
-        let mut s = String::new();
-        self.format_to_write(&mut s, value)
-            .expect("Failed to write to a String.");
-        s
+        self.0.format_to_string(value)
     }
 }
